@@ -178,8 +178,13 @@ char* strsep(char** str, const char* delims)
 }
 #endif
 
-/* does the same thing as 'mkdir -p' */
 int _alpm_makepath(const char *path)
+{
+	return(_alpm_makepath_mode(path, 0755));
+}
+
+/* does the same thing as 'mkdir -p' */
+int _alpm_makepath_mode(const char *path, mode_t mode)
 {
 	char *orig, *str, *ptr;
 	char full[PATH_MAX] = "";
@@ -196,7 +201,7 @@ int _alpm_makepath(const char *path)
 			strcat(full, "/");
 			strcat(full, ptr);
 			if(stat(full, &buf)) {
-				if(mkdir(full, 0755)) {
+				if(mkdir(full, mode)) {
 					FREE(orig);
 					umask(oldmask);
 					_alpm_log(PM_LOG_ERROR, _("failed to make path '%s' : %s\n"),
@@ -399,6 +404,8 @@ int _alpm_unpack(const char *archive, const char *prefix, const char *fn)
 		
 		if(S_ISREG(st->st_mode)) {
 			archive_entry_set_mode(entry, 0644);
+		} else if(S_ISDIR(st->st_mode)) {
+			archive_entry_set_mode(entry, 0755);
 		}
 
 		if (fn && strcmp(fn, entryname)) {
@@ -582,9 +589,7 @@ const char *_alpm_filecache_setup(void)
 	}
 
 	/* we didn't find a valid cache directory. use /tmp. */
-	i = alpm_option_get_cachedirs();
 	tmp = alpm_list_add(NULL, strdup("/tmp/"));
-	FREELIST(i);
 	alpm_option_set_cachedirs(tmp);
 	_alpm_log(PM_LOG_DEBUG, "using cachedir: %s", "/tmp/\n");
 	_alpm_log(PM_LOG_WARNING, _("couldn't create package cache, using /tmp instead\n"));
